# Configuring Socket.IO

Socket.IO can be configured in the same fashion as you would do with your
Express application. It allows the usage of **configure**, **set**, **enable**
and **disable**.

The following options can be configured.

### Server

* **origins** defaults to `*:*`
 * The origins that are allowed to connect to
  the Socket.IO server.

* **store** defaults to `MemoryStore`
 * A store instance that is used to for
  client storage logic. See
  [MemoryStore](https://github.com/LearnBoost/Socket.IO/blob/master/lib/stores/memory.js)
  for working implementation. The MemoryStore only allows you deploy socket.io
  on a single process. If you want to scale to multiple process and / or
  multiple servers you can use our `RedisStore` which uses the Redis NoSQL
  database as man in the middle.

* **logger** defaults to `Logger`
 * A logger instance that is used to output
  log information. See
  [Logger](https://github.com/LearnBoost/Socket.IO/blob/master/lib/logger.js)
  for working implementation.

* **static** defaults to `Static`
 * A Static instance that is used to serve the
  socket.io client and it's dependencies. See
  [Static](https://github.com/LearnBoost/socket.io/blob/master/lib/static.js)
  for working implementation.

* **heartbeats** defaults to `true`
 * Should we use heartbeats to check the
 health of Socket.IO connections.

* **resource** defaults to `/socket.io`
 * The begin point where Socket.IO
 starts looking for incoming connections.  This should be the same between the
 client and the server.

* **transports** defaults to `websocket, htmlfile, xhr-polling, jsonp-polling`
 * A array of allowed transport methods. The flashsocket transport is disabled
 by default, and can be enabled by setting the transports with the
 `flashsocket` option included.

* **authorization** defaults to `false` * Global authorization for Socket.IO
 access, this is called when the initial handshake is performed with the
 server. See [Authorization and
 handshaking](https://github.com/LearnBoost/socket.io/wiki/Authorizing) for
 detailed information.

* **log level** defaults to `3`
 * The amount of detail that the server should output to the logger.

* **close timeout** defaults to `25` seconds
 * The timeout for the client, when it closes the connection it still X amounts
 of seconds to do re open of the connection. This value is sent to the client
 after a successful handshake.

* **heartbeat timeout** defaults to `15` seconds
 * The timeout for the client when it should send a new heart beat to the
 server. This value is sent to the client after a successful handshake.

* **heartbeat interval** defaults to `20` seconds
 * The timeout for the server, we should receive a heartbeat from the client
 within this interval. This should be greater than the `heartbeat timeout`.

* **polling duration** defaults to `20` seconds
 * The maximum duration of one HTTP poll, if it exceeds this limit it will be
 closed.

* **flash policy server** defaults to `true`
 * Enable the flash policy server if the flashsocket transport is enabled.

* **flash policy port** defaults to `10843`
 * By default the Socket.IO client will check port 10843 on your server to see
 if flashsocket connections are allowed. The Adobe Flash Player normally uses
 843 as default port, but we decided to default to a non root port.

* **destroy upgrade** defaults to `true`
 * Do we need to destroy non-socket.io upgrade requests.

* **browser client** defaults to `true`
 * Does Socket.IO need to serve the static resources like socket.io.js and
 WebSocketMain.swf etc.

* **browser client cache** defaults to `true`
 * Cache the Socket.IO file generation in the memory of the process to speed up
 the serving of the static files.

* **browser client minification** defaults to `false`
 * Does Socket.IO need to send a minified build of socket.io.js.

* **browser client etag** defaults to `false`
 * Does Socket.IO need to send a ETag header for the static requests.

* **browser client gzip** defaults to `false`
 * Does Socket.IO need to GZIP the static files. This process is only done once
 and the computed output is stored in memory. So we don't have to spawn a
 `gzip` process for each request.

* **browser client handler** defaults to `false`
 * A function that should serve all static handling, including socket.io.js et
 al.

* **match origin protocol** defaults to `false`
 * Meant to be used when running socket.io behind a proxy. Should be set to true when you want the location handshake to match the protocol of the origin. This fixes issues with terminating the SSL in front of Node and forcing location to think it's wss instead of ws.

### Client

* **resource** defaults to `socket.io`
  * Note the subtile diff difference between the server, this is one is missing
  a `/`. These 2 should be in sync with the server to prevent miss matches.

* **connect timeout** defaults to `10000` ms
 * How long should Socket.IO wait before it aborts the connection attempt with
 the server to try another fall-back. Please note that some transports require a
 longer timeout than others. Setting this really low could potentially harm
 them.

* **try multiple transports** defaults to `true`
 * When Socket.IO reconnects and it keeps failing over and over again, should
 it try all available transports when it finally gives up.

* **reconnect** defaults to `true`
 * Should Socket.IO automatically reconnect when it detects a dropped
 connection or timeout.

* **reconnection delay** defaults to `500` ms
  * The initial timeout to start a reconnect, this is increased using a
  exponential back off algorithm each time a new reconnection attempt has been
  made.

* **reconnection limit** defaults to `Infinity`
  * The maximum reconnection delay in milliseconds, or Infinity.

* **max reconnection attempts** defaults to `10`
  * How many times should Socket.IO attempt to reconnect with the server after a
  a dropped connection. After this we will emit the `reconnect_failed` event.

* **sync disconnect on unload** defaults to `true`
 * Do we need to send a disconnect packet to server when the browser unloads.

* **auto connect** defaults to `true`
 * When you call `io.connect()` should Socket.IO automatically establish a
 connection with the server.

* **flash policy port** defaults to `10843`
 * If you have Flashsocket enabled, this should match the same port as the
 server.

* **force new connection** defaults to `false`
 * Force multiple io.connect() calls to the same server to use different
 connections.

> ### The browser client handler function
> 
> This function receives 2 arguments:
> 
> 1. `HTTPRequest` req The request object
> 2. `HTTPResponse` res The response object
> 
> The client handle should take care of all the handling of requests, this
> includes caching and error handling.

## Understanding the configure method

The Socket.IO configuration method mimics the same behavior that you might know
from the Express web framework. It allows you to configure Socket.IO based on
`NODE_ENV` environment flags such as _production_ and _development_.

When the `configure()` is called without an environment flag it will run in
every environment. In side the configure function you can use the **set**,
**enable** and **disable**. The example below will enable the Flashsocket and
etag for `production` and only allow WebSocket connection when developing.

```js
var io = require('socket.io').listen(80);

io.configure('production', function(){
  io.enable('browser client etag');
  io.set('log level', 1);

  io.set('transports', [
    'websocket'
  , 'flashsocket'
  , 'htmlfile'
  , 'xhr-polling'
  , 'jsonp-polling'
  ]);
});

io.configure('development', function(){
  io.set('transports', ['websocket']);
});
```

### How do you set the NODE_ENV flag

The following example will set the `NODE_ENV` flag to _production_.
```
NODE_ENV=production node app.js
```

## Advised production settings

Socket.IO is configured to make the development process easier, thats why
certain settings like `browser client etag` is set to false by default. This
will, for example prevent caching on the socket.io.js file.

The following options advised to be set in production:

```js
io.enable('browser client minification');  // send minified client
io.enable('browser client etag');          // apply etag caching logic based on version number
io.enable('browser client gzip');          // gzip the file
io.set('log level', 1);                    // reduce logging
io.set('transports', [                     // enable all transports (optional if you want flashsocket)
    'websocket'
  , 'flashsocket'
  , 'htmlfile'
  , 'xhr-polling'
  , 'jsonp-polling'
]);
```

Heroku has provided their own set of [recommended set of options](http://devcenter.heroku.com/articles/using-socket-io-with-node-js-on-heroku).